package fr.unice.javinspector.GUI;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import fr.unice.javinspector.core.analyser.*;
import javax.swing.border.*;
import fr.unice.javinspector.core.search.*;
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.ArrayList;
import java.awt.Image;
import fr.unice.javinspector.MVC.manager.Manager;
import fr.unice.javinspector.MVC.manager.SplitPaneTopManager;
import fr.unice.javinspector.MVC.manager.AbstractTopManager;
import fr.unice.javinspector.MVC.controler.TreeHeritageControler;
import fr.unice.javinspector.MVC.view.TreeHeritageView;
import fr.unice.javinspector.GUI.util.JTabbedPaneWithCloseIcons;
import fr.unice.javinspector.GUI.search.*;
import fr.unice.javinspector.core.XML.*;
import fr.unice.javinspector.GUI.main.*;

import java.io.File;
import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;

/**
 * <p>Title: Class JavInspectorGUI</p>
 * <p>Description: The class that build the main part of the JavInspector graphic user interface.</p>
 * <p>Copyright: UNICE</p>
 * @author Charbit Lucas - Gahide Jrome - Galbois Xavier - Zerbib Gabriel
 * @version 1.0
 */

public class JavInspectorGUI extends JFrame
    implements ActionListener, ChangeListener {
  // to show/hide the searches results
  private boolean SEARCH_VIEW = false;
  // fot the main split pane
  private int lastDividerLocation;
  // the hashtable of manager, one manager for each inspector
  private Hashtable managers;
  // the hashtable of inspector
  private Hashtable inspector;
  // the classFileChooser to open new classifiers
  private ClassFileChooser classFileChooser;
  // the starting panel
  private ExplicationPanel explicationPanel = new ExplicationPanel();
  // the search window
  private SearchWindow searchWindow;
  // the splitpane for the searches results
  private JSplitPane jSplitPane;
  // the panel for the searches results
  private JPanel panelSearch = new JPanel();
  // the tabbedPane for the searches results
  protected JTabbedPaneWithCloseIcons jTabbedPaneSearch = new
      JTabbedPaneWithCloseIcons();
  // the main panel of the interface
  private JPanel globalPanel = new JPanel();
  // the plugin menu
  private PluginMenu pluginMenu = new PluginMenu();
  // ***** OTHER OBJETS *****
  JPanel contentPane;
  JMenuBar jMenuBarGlobal = new JMenuBar();
  JMenu jMenuFile = new JMenu();
  JMenuItem jMenuFileOpen = new JMenuItem();
  JToolBar jToolBar = new JToolBar();
  JButton jButtonOpen = new JButton();
  JButton jButtonGlobalSearch = new JButton();
  ImageIcon openImage = new ImageIcon(JavInspectorGUI.class.getResource(
      "/imgs/open.gif"));
  ImageIcon globalSearchImage = new ImageIcon(JavInspectorGUI.class.getResource(
      "/imgs/globalsearch.gif")); ;
  ImageIcon searchImage = new ImageIcon(JavInspectorGUI.class.getResource(
      "/imgs/search.gif"));
  ImageIcon closeImage = new ImageIcon(JavInspectorGUI.class.getResource(
      "/imgs/close.gif"));
  ImageIcon viewSearch = new ImageIcon(JavInspectorGUI.class.getResource(
      "/imgs/viewsearch.gif"));
  BorderLayout borderLayout1 = new BorderLayout();
  JMenuItem jMenuFileClose = new JMenuItem();
  JMenuItem jMenuFileExit = new JMenuItem();
  Border border1;
  TitledBorder titledBorder1;
  JPanel jPanelCenter = new JPanel();
  JPanel jPanelSatut = new JPanel();
  FlowLayout flowLayout1 = new FlowLayout();
  FlowLayout flowLayoutCenter = new FlowLayout(FlowLayout.CENTER);
  JComboBox jComboBoxInspectors = new JComboBox();
  TitledBorder titledBorder2;
  TitledBorder titledBorder3;
  Border border2;
  JButton jButtonClose = new JButton();
  CardLayout cardLayout = new CardLayout();
  Border border3;
  JLabel jLabel1 = new JLabel();
  Border border4;
  JButton jButtonViewSearch = new JButton();
  BorderLayout borderLayout2 = new BorderLayout();
  FlowLayout flowLayout2 = new FlowLayout();
  JMenuItem jMenuITransformXML = new JMenuItem();

  /**
   * the constructor.
   */
  public JavInspectorGUI() {
    enableEvents(AWTEvent.WINDOW_EVENT_MASK);
    try {
      jbInit();
      jToolBar.setRollover(true);
    }
    catch (Exception e) {
      e.printStackTrace();
    }
    managers = new Hashtable();
    inspector = new Hashtable();
    // a reference to the frame is given, to allow these JDialogs objects to appear
    // in the center of the frame
    classFileChooser = new ClassFileChooser(this);
    searchWindow = new SearchWindow(this);
    // we listen to the comboBox to allow inspector swapping
    jComboBoxInspectors.addActionListener(this);
    jComboBoxInspectors.setActionCommand("SWAP_INSPECTOR");
    // we place the first panel
    JScrollPane js = new JScrollPane(explicationPanel);
    jPanelCenter.add("explicationPanel", js);
    // we show the start panel in the panelCenter
    cardLayout.show(jPanelCenter, "explicationPanel");
    // we add the plugin menu to the menu bar
    jMenuBarGlobal.add(pluginMenu);
    // we update the menu to disable/enable icons
    updateMenu();
    // set the tab of this result search pane at bottom
    jTabbedPaneSearch.setTabPlacement(JTabbedPane.BOTTOM);
  }

  public Inspector getCurrentInspector() {
    String inspectorName = (String)jComboBoxInspectors.getSelectedItem();
    Inspector currentI = (Inspector)inspector.get(inspectorName);
    return currentI;
  }

  /**
   * This function looks if the toolbar buttons actions can be performed, if so,
   * the buttons are enabled, otherwise, they're disabled.
   */
  public void updateMenu() {
    if ((inspector == null) || (inspector.size() < 1)) {
      jButtonClose.setEnabled(false);
      jButtonGlobalSearch.setEnabled(false);
      jButtonViewSearch.setEnabled(false);
      jMenuFileClose.setEnabled(false);
    }
    else {
      jButtonClose.setEnabled(true);
      jButtonGlobalSearch.setEnabled(true);
      jButtonViewSearch.setEnabled(true);
      jMenuFileClose.setEnabled(true);
    }
  }

  /**
   * this function remove a manager and his related inspector from their hashtables.
   * @param currentInspector : the inspector related to the manager we want to close.
   */
  public void removeManager(Inspector currentInspector) {
    Manager currentManager = (Manager) managers.get(currentInspector.
        getClassifierName());
    managers.remove(currentManager);
    jPanelCenter.remove(currentManager.getViewContainer());
    inspector.remove(currentInspector.getClassifierName());
    // we remove the comboBox entry for this inspector
    jComboBoxInspectors.removeItem(jComboBoxInspectors.getSelectedItem());
    if (inspector.size() == 0) {
      cardLayout.show(jPanelCenter, "explicationPanel");
      SEARCH_VIEW = false;
      showResultPanel(false);
      jTabbedPaneSearch.removeAll();
    }
    updateMenu();
  }

  /**
   * To start a search on an opened classifierAnalyser.
   * PERSONNE APELLE CETTE FONCTION 
   * @param e : the target clasifierAnalyser.
   */
  public void stateChanged(ChangeEvent e) {
    if(e.getSource() == jTabbedPaneSearch) {
      if(jTabbedPaneSearch.getTabCount() == 0) {
        showResultPanel();
      }
    }
  }

  /**
   * To show or hide the ResultPanel, make the split pane appear or disappear.
   * @param yesNo : yes to make the split pane visible, or no to hide it if still visible.
   */
  public void showResultPanel(boolean yesNo) {
    if (yesNo && !SEARCH_VIEW) {
      jSplitPane.setTopComponent(jPanelCenter);
      jSplitPane.setBottomComponent(jTabbedPaneSearch);
      jSplitPane.setDividerLocation(lastDividerLocation);
      this.getContentPane().remove(jPanelCenter);
      this.getContentPane().add(globalPanel, BorderLayout.CENTER);
      this.getContentPane().setVisible(false);
      this.getContentPane().setVisible(true);
      SEARCH_VIEW = true;
    }
    else if (!yesNo && !SEARCH_VIEW) {
      lastDividerLocation = jSplitPane.getDividerLocation();
      this.getContentPane().remove(globalPanel);
      this.getContentPane().add(jPanelCenter, BorderLayout.CENTER);
      this.getContentPane().setVisible(false);
      this.getContentPane().setVisible(true);
      SEARCH_VIEW = false;
    }
  }

  public void showResultPanel() {
    if (!SEARCH_VIEW) {
      jSplitPane.setTopComponent(jPanelCenter);
      jSplitPane.setBottomComponent(jTabbedPaneSearch);
      jSplitPane.setDividerLocation(lastDividerLocation);
      this.getContentPane().remove(jPanelCenter);
      this.getContentPane().add(globalPanel, BorderLayout.CENTER);
      this.getContentPane().setVisible(false);
      this.getContentPane().setVisible(true);
      SEARCH_VIEW = true;
    }
    else {
      lastDividerLocation = jSplitPane.getDividerLocation();
      this.getContentPane().remove(globalPanel);
      this.getContentPane().add(jPanelCenter, BorderLayout.CENTER);
      this.getContentPane().setVisible(false);
      this.getContentPane().setVisible(true);
      SEARCH_VIEW = false;
    }
  }

  /**
   * This function is called when a listened widget fire an event.
   * If the widget is the combo box, the selected inspector has changed, and the
   * displayed manager must be changed.
   * @param e : the event fired.
   */
  public void actionPerformed(ActionEvent e) {
    // if so, the event comes from the combo box, we change the current project
    if (e.getActionCommand().compareTo("SWAP_INSPECTOR") == 0) {
      String inspectorName = (String) jComboBoxInspectors.getSelectedItem();
      cardLayout.show(jPanelCenter, inspectorName);
    }
  }

  /**
   * This function is called when the File->Open menu has been selected.
   * @param e : the action event.
   */
  public void jMenuFileOpen_actionPerformed(ActionEvent e) {
    jButtonOpen_actionPerformed(e);
  }

  /**
   * To quit the application when the window is closed.
   * @param e : the windonEvent.
   */
  protected void processWindowEvent(WindowEvent e) {
    super.processWindowEvent(e);
    if (e.getID() == WindowEvent.WINDOW_CLOSING) {
      jMenuFileExit_actionPerformed(null);
    }
  }

  /**
   * This function search in the inspector hashtable if an inspector fot the specified
   * class is still opened.
   * @param keyClass : the class
   * @return : returns true if the inspector exist, false other nether.
   */
  public boolean lookForInspector(Class keyClass) {
    Enumeration enum = inspector.elements();
    while (enum.hasMoreElements()) {
      if ( ( (Inspector) enum.nextElement()).getClassInspected() == keyClass) {
        return true;
      }
    }
    return false;
  }

  /**
   * This function is called when the Open button in the toolbar have been clicked.
   * This make the classFileChooser appear, and wait for the user to select a valid class
   * to load. Once it's done, a new inspector and a new manager for this inspector are created.
   * @param e : the action event.
   */
  void jButtonOpen_actionPerformed(ActionEvent e) {
    classFileChooser.show(this);
    Class keyClass = classFileChooser.getSelectedClass();
    if (keyClass != null) {
      if (lookForInspector(keyClass) == true) {
        JOptionPane.showMessageDialog(this,
                                      "Sorry but the class " + keyClass +
                                      " is still opened.",
                                      "Inspector information message",
                                      JOptionPane.INFORMATION_MESSAGE);
      }
      else {
        try {
          Inspector newInspector = new Inspector(keyClass);
          // we create a new AbstractTopManager fot the new created inspector, in our
          // case it's a SplitPaneTopManager
          AbstractTopManager newManagerGUI = new SplitPaneTopManager(
              newInspector, pluginMenu.getPluginSettings());
          String inspectorName = newInspector.getClassifierName();
          inspector.put(inspectorName, newInspector);
          managers.put(inspectorName, newManagerGUI);
          // we add a new entry in the inspector combo box
          jComboBoxInspectors.addItem(inspectorName);
          jComboBoxInspectors.setSelectedItem(inspectorName);
          // we show the panel of the new created manager in the main panel
          jPanelCenter.add(inspectorName, newManagerGUI.getViewContainer());
          cardLayout.show(jPanelCenter, inspectorName);
        }
        // if we can't create the inspector, we warm the user
        catch (NoClassDefFoundError ex) {
          JOptionPane.showMessageDialog(this,
              "Unable to create an inspector panel for " +
              "the classifier " + keyClass.getName(),
              "Inspector error message",
              JOptionPane.ERROR_MESSAGE);
          return;
        }
      }
      updateMenu();
    }
  }

  /**
   * this function is called when the Close button of the toolbar have been clicked.
   * This function remove the currently selected inspector and his manager from their hashtable.
   * @param e : the action event
   */
  void jButtonClose_actionPerformed(ActionEvent e) {
    String inspectorName = (String) jComboBoxInspectors.getSelectedItem();
    Inspector currentInspector = (Inspector) inspector.get(inspectorName);
    removeManager(currentInspector);
    updateMenu();
  }

  /**
   * this function is called when the File->Close inspector menu is selected.
   * Has the same than jButtonClose_actionPerformed.
   * @param e : the action event
   */
  void jMenuFileClose_actionPerformed(ActionEvent e) {
    String inspectorName = (String) jComboBoxInspectors.getSelectedItem();
    Inspector currentInspector = (Inspector) inspector.get(inspectorName);
    removeManager(currentInspector);
    updateMenu();
  }

  /**
   * this function is called when the File->Exit menu is selected.
   * Close the application
   * @param e : the action event
   */
  void jMenuFileExit_actionPerformed(ActionEvent e) {
    System.exit(0);
  }

  /**
   * This function is called when the viewSearch button of the toolbar have been clicked.
   * @param e : the action event.
   */
  void jButtonViewSearch_actionPerformed(ActionEvent e) {
    showResultPanel();
  }

  public JPanel getPanelSearch() {
    return panelSearch;
  }

  /**
   * this function is called when the globalSearch button of the toolbar have been clicked.
   * This function make the search window appear.
   * @param e : the action event
   */
  void jButtonGlobalSearch_actionPerformed(ActionEvent e) {

    Inspector inspectorCurrent = getCurrentInspector();
// old Jjay
//    Inspector inspecTemp = (Inspector) inspector.get( (String)
//        jComboBoxInspectors.getSelectedItem());
//    searchWindow.setClassifierAnalyser(inspecTemp.getMainClassifier());
    searchWindow.setInspector(inspectorCurrent);
    searchWindow.show(this);
  }

  public JTabbedPaneWithCloseIcons getJTabbedPaneSearch() {
    return jTabbedPaneSearch;
  }

  /**
   * this function is called when the File->Save file XML menu have been selected.
   * @todo gab doc
   * @param e
   */
  void jMenuITransformXML_actionPerformed(ActionEvent e) {
    JFileChooser fileChooser = new JFileChooser();
    fileChooser.setFileFilter(new XMLFilter());
    fileChooser.setDialogType(JFileChooser.OPEN_DIALOG);
    int choix = fileChooser.showOpenDialog(this);
    if (choix == fileChooser.APPROVE_OPTION) {
      File file = fileChooser.getSelectedFile();
      if (file == null) {
        JOptionPane.showMessageDialog(this, "this file do not exist");
      }
      else if (!file.exists()) {
        JOptionPane.showMessageDialog(this, "this file do not exist");
      }
      else {
        try {
          /**
           * @todo virer ca c du dur!!!!!!
           */
          File xsl = new File("xml/essai.xsl");
          if (xsl == null || !xsl.exists()) {
            JOptionPane.showMessageDialog(this,
                                          "make sure that the xsl file exists");
            return;
          }
          File txt = new File(file.getAbsolutePath() + ".txt");
          if (TransformFile.transform(file, txt, xsl) == true){
            JOptionPane.showMessageDialog(this, "your file was transform in "
                                          + txt.getName());
          }
        }
        catch (Exception ex) {
          JOptionPane.showMessageDialog(this, "error while saving");
          return;
        }
      }
    }
    else {
    }
  }

  /**
   * GUI initialisation
   * @throws Exception
   */
  private void jbInit() throws Exception {
    contentPane = (JPanel)this.getContentPane();
    border1 = BorderFactory.createEmptyBorder(0, 2, 0, 2);
    border2 = BorderFactory.createEmptyBorder(0, 2, 0, 2);
    border3 = BorderFactory.createBevelBorder(BevelBorder.LOWERED, Color.white,
                                              Color.white, Color.darkGray,
                                              Color.gray);
    border4 = BorderFactory.createEmptyBorder(0, 2, 0, 2);
    contentPane.setLayout(borderLayout1);
    this.setSize(new Dimension(514, 300));
    this.setTitle("JavInspector");
    jMenuFile.setText("File");
    jMenuFileOpen.setActionCommand("OpenClassifier");
    jMenuFileOpen.setText("Open classifier ...");
    jMenuFileOpen.addActionListener(new
                                    JavInspectorGUI_jMenuFileOpen_ActionAdapter(this));
    jButtonOpen.setIcon(openImage);
    jButtonOpen.setSelected(true);
    jButtonOpen.setText("");
    jButtonOpen.addActionListener(new JavInspectorGUI_jButtonOpen_actionAdapter(this));
    jButtonOpen.setMaximumSize(new Dimension(30, 24));
    jButtonOpen.setMinimumSize(new Dimension(30, 24));
    jButtonOpen.setPreferredSize(new Dimension(30, 24));
    jButtonOpen.setToolTipText("Click here to open a new classifier and start a new inspection ");
    jButtonOpen.setActionCommand("Button1");
    jButtonGlobalSearch.setIcon(globalSearchImage);
    jButtonGlobalSearch.setText("");
    jButtonGlobalSearch.addActionListener(new
        JavInspectorGUI_jButtonGlobalSearch_actionAdapter(this));
    jButtonGlobalSearch.setMaximumSize(new Dimension(30, 24));
    jButtonGlobalSearch.setMinimumSize(new Dimension(30, 24));
    jButtonGlobalSearch.setPreferredSize(new Dimension(30, 24));
    jButtonGlobalSearch.setToolTipText("Launch a search on the current inspector");
    jMenuFileClose.setActionCommand("CloseInspector");
    jMenuFileClose.setText("Close inspector ...");
    jMenuFileClose.addActionListener(new JavInspectorGUI_jMenuFileClose_actionAdapter(this));
    jMenuFileExit.setActionCommand("ExitJavInspector");
    jMenuFileExit.setText("Exit JavInspector");
    jMenuFileExit.addActionListener(new
                                    JavInspectorGUI_jMenuFileExit_actionAdapter(this));
    jToolBar.setFont(new java.awt.Font("Dialog", 1, 10));
    jToolBar.setBorder(null);
    jToolBar.setFloatable(false);
    jPanelCenter.setLayout(cardLayout);
    jPanelSatut.setBorder(BorderFactory.createEtchedBorder());
    jPanelSatut.setMinimumSize(new Dimension(0, 0));
    jPanelSatut.setLayout(flowLayout1);
    flowLayout1.setAlignment(FlowLayout.LEFT);
    flowLayout1.setVgap(2);
    jComboBoxInspectors.setBackground(Color.white);
    jComboBoxInspectors.setBorder(border3);
    jComboBoxInspectors.setDebugGraphicsOptions(0);
    jComboBoxInspectors.setMaximumSize(new Dimension(200, 20));
    jComboBoxInspectors.setMinimumSize(new Dimension(26, 20));
    jComboBoxInspectors.setPreferredSize(new Dimension(85, 20));
    jComboBoxInspectors.setToolTipText("Change the current inspector");
    jButtonClose.setIcon(closeImage);
    jButtonClose.setMaximumSize(new Dimension(30, 24));
    jButtonClose.setMinimumSize(new Dimension(30, 24));
    jButtonClose.setPreferredSize(new Dimension(30, 24));
    jButtonClose.setToolTipText("Close the current inspector");
    jButtonClose.addActionListener(new
                                   JavInspectorGUI_jButtonClose_actionAdapter(this));
    jLabel1.setBorder(border4);
    jLabel1.setText("Inspectors ");
    jButtonViewSearch.setBorder(null);
    jButtonViewSearch.setMaximumSize(new Dimension(30, 24));
    jButtonViewSearch.setMinimumSize(new Dimension(30, 24));
    jButtonViewSearch.setPreferredSize(new Dimension(30, 24));
    jButtonViewSearch.setToolTipText("View searches results");
    jButtonViewSearch.setIcon(viewSearch);
    jButtonViewSearch.addActionListener(new
        JavInspectorGUI_jButtonViewSearch_actionAdapter(this));
    contentPane.setBorder(null);
    contentPane.setMinimumSize(new Dimension(0, 0));
    panelSearch.setBorder(null);
    panelSearch.setMaximumSize(new Dimension(900, 900));
    globalPanel.setBorder(null);
    jMenuITransformXML.setText("Transform XML file");
    jMenuITransformXML.addActionListener(new
        JavInspectorGUI_jMenuITransformXML_actionAdapter(this));
    jPanelCenter.setMinimumSize(new Dimension(0, 0));
    jTabbedPaneSearch.setToolTipText("Search results");
    jToolBar.add(jButtonOpen);
    jToolBar.addSeparator();
    jToolBar.add(jButtonGlobalSearch);
    jToolBar.add(jButtonViewSearch, null);
    jToolBar.addSeparator();
    jToolBar.add(jLabel1, null);
    jMenuFile.add(jMenuFileOpen);
    jMenuFile.add(jMenuFileClose);
    jMenuFile.addSeparator();
    jMenuFile.add(jMenuITransformXML);
    jMenuFile.addSeparator();
    jMenuFile.add(jMenuFileExit);
    jMenuBarGlobal.add(jMenuFile);
    this.setJMenuBar(jMenuBarGlobal);
    contentPane.add(jToolBar, BorderLayout.NORTH);
    contentPane.add(jPanelCenter, BorderLayout.CENTER);
    jPanelCenter.add(jPanelSatut, "jPanelSatut");
    jToolBar.add(jComboBoxInspectors, null);
    jToolBar.add(jButtonClose, null);
    jToolBar.addSeparator();
    // the search splitpane
    jSplitPane = new JSplitPane();
    jSplitPane.setOrientation(JSplitPane.VERTICAL_SPLIT);
    lastDividerLocation = 300;
    jSplitPane.setDividerLocation(lastDividerLocation);
    jSplitPane.setLastDividerLocation(500);
    globalPanel.setLayout(new BorderLayout());
    globalPanel.add(jSplitPane, BorderLayout.CENTER);
    jSplitPane.setDividerLocation(lastDividerLocation);
    panelSearch.add(new JLabel("ICI CONNARD"));
    jTabbedPaneSearch.addChangeListener(this);
  }
}

// Intern classes
class JavInspectorGUI_jMenuFileOpen_ActionAdapter
    implements ActionListener {
  JavInspectorGUI adaptee;

  JavInspectorGUI_jMenuFileOpen_ActionAdapter(JavInspectorGUI adaptee) {
    this.adaptee = adaptee;
  }

  public void actionPerformed(ActionEvent e) {
    adaptee.jMenuFileOpen_actionPerformed(e);
  }
}

class JavInspectorGUI_jButtonOpen_actionAdapter
    implements java.awt.event.ActionListener {
  JavInspectorGUI adaptee;

  JavInspectorGUI_jButtonOpen_actionAdapter(JavInspectorGUI adaptee) {
    this.adaptee = adaptee;
  }

  public void actionPerformed(ActionEvent e) {
    adaptee.jButtonOpen_actionPerformed(e);
  }
}

class JavInspectorGUI_jButtonClose_actionAdapter
    implements java.awt.event.ActionListener {
  JavInspectorGUI adaptee;

  JavInspectorGUI_jButtonClose_actionAdapter(JavInspectorGUI adaptee) {
    this.adaptee = adaptee;
  }

  public void actionPerformed(ActionEvent e) {
    adaptee.jButtonClose_actionPerformed(e);
  }
}

class JavInspectorGUI_jMenuFileExit_actionAdapter
    implements java.awt.event.ActionListener {
  JavInspectorGUI adaptee;

  JavInspectorGUI_jMenuFileExit_actionAdapter(JavInspectorGUI adaptee) {
    this.adaptee = adaptee;
  }

  public void actionPerformed(ActionEvent e) {
    adaptee.jMenuFileExit_actionPerformed(e);
  }
}

class JavInspectorGUI_jButtonViewSearch_actionAdapter
    implements java.awt.event.ActionListener {
  JavInspectorGUI adaptee;

  JavInspectorGUI_jButtonViewSearch_actionAdapter(JavInspectorGUI adaptee) {
    this.adaptee = adaptee;
  }

  public void actionPerformed(ActionEvent e) {
    adaptee.jButtonViewSearch_actionPerformed(e);
  }
}

class JavInspectorGUI_jButtonGlobalSearch_actionAdapter
    implements java.awt.event.ActionListener {
  JavInspectorGUI adaptee;

  JavInspectorGUI_jButtonGlobalSearch_actionAdapter(JavInspectorGUI adaptee) {
    this.adaptee = adaptee;
  }

  public void actionPerformed(ActionEvent e) {
    adaptee.jButtonGlobalSearch_actionPerformed(e);
  }
}

class JavInspectorGUI_jMenuITransformXML_actionAdapter
    implements java.awt.event.ActionListener {
  JavInspectorGUI adaptee;

  JavInspectorGUI_jMenuITransformXML_actionAdapter(JavInspectorGUI adaptee) {
    this.adaptee = adaptee;
  }

  public void actionPerformed(ActionEvent e) {
    adaptee.jMenuITransformXML_actionPerformed(e);
  }
}

class JavInspectorGUI_jMenuFileClose_actionAdapter implements java.awt.event.ActionListener {
  JavInspectorGUI adaptee;

  JavInspectorGUI_jMenuFileClose_actionAdapter(JavInspectorGUI adaptee) {
    this.adaptee = adaptee;
  }
  public void actionPerformed(ActionEvent e) {
    adaptee.jMenuFileClose_actionPerformed(e);
  }
}
