package fr.unice.javinspector.GUI.main;

import java.io.*;
import java.util.*;

import java.awt.event.*;
import javax.swing.*;

import fr.unice.javinspector.plugin.*;
import fr.unice.javinspector.core.loader.*;
import fr.unice.javinspector.MVC.*;
import fr.unice.javinspector.MVC.view.TreeHeritageView;
import fr.unice.javinspector.MVC.controler.TreeHeritageControler;
import fr.unice.javinspector.MVC.view.GraphDetailView;
import fr.unice.javinspector.MVC.controler.GraphDetailControler;
import fr.unice.javinspector.MVC.manager.AbstractTopManager;
import fr.unice.javinspector.MVC.view.MyCompositionView;
import fr.unice.javinspector.MVC.controler.MyCompositionControler;

/**
 * <p>Title: JavInspector</p>
 * <p>Description: A dynamic menu build from files present in a directory
 * and from the different plugin types</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: </p>
 * @author Jerme GAHIDE - Lucas CHARBIT - Gabriel ZERBIB - Xavier GALBOIS
 * @version 1.0
 */

public class PluginMenu extends JMenu implements ActionListener{
  private PluginLoader pClassLoader = null;
  private Hashtable plugins = null;
  private JMenu controlerMenu;
  private JMenu viewMenu;
  private JMenuItem rescanItem;
  private Hashtable radioMenus;
  private static final String RESCAN_COMMAND = "rescan";

  /**
   * Default construcutor constructs the plugin menu
   */
  public PluginMenu() {
    super("Plugins");
    pClassLoader = new PluginLoader("plugins/", null);
    radioMenus = new Hashtable();
    controlerMenu = new JMenu("Controlers");
    viewMenu = new JMenu("Views");
    rescanItem = new JMenuItem("Rescan");
    rescanItem.setActionCommand(RESCAN_COMMAND);
    rescanItem.addActionListener(this);
    init();
  }

  private void init(){
    this.add(controlerMenu);
    this.add(viewMenu);
    this.add(rescanItem);
    insertApplicationPlugins();
    try {
      plugins = pClassLoader.loadPlugins();
      insertPluginsInMenu(this, plugins);
    }
    catch (FileNotFoundException ex) {
      JOptionPane.showMessageDialog(this.getTopLevelAncestor(),
                                    "Failed to load Plugins",
                                    "FileNotFoundException",
                                    JOptionPane.ERROR_MESSAGE);
    }

  }

  /**
   *
   * @param e
   */
  public void actionPerformed(ActionEvent e){
      if(e.getActionCommand().compareTo(RESCAN_COMMAND) == 0){
        radioMenus.clear();
        controlerMenu.removeAll();
        viewMenu.removeAll();
        init();
      }
  }


  /**
   * inserts the Plugins provided with this application (they are not in the
   * plugin directory)
   */
  private void insertApplicationPlugins(){
    addPluginRadioButton(TreeHeritageView.class);
    addPluginRadioButton(TreeHeritageControler.class);
    addPluginRadioButton(GraphDetailView.class);
    addPluginRadioButton(GraphDetailControler.class);
    addPluginRadioButton(MyCompositionView.class);
    addPluginRadioButton(MyCompositionControler.class);
  }

  /**
   * inserts the plugin found
   * @param topMenu
   * @param plugins the plugins to insert
   */
  private void insertPluginsInMenu(JMenu topMenu, Hashtable  plugins){
    int type;

    Iterator iter =  plugins.keySet().iterator();
    while (iter.hasNext()) {
      String item = (String) iter.next();
      Class cl = (Class) plugins.get(item);
      type = getType(cl);
      if (type >= 0){
        addPluginRadioButton(type, cl, cl.getName());
      }
    }
  }

  /**
   * adds this class(a Plugin) to a menu
   * @param cl the class
   */
  private void addPluginRadioButton(Class cl){
    addPluginRadioButton(getType(cl), cl, cl.getName());
  }

  /**
   * adds this class(a Plugin) to a menu depending on its type
   * @param type the type of the class
   * @param cl the class
   * @param name the name to appear in the menu
   */
  public void addPluginRadioButton(int type, Class cl, String name){
    RadioStruct rstruct = null;
    if ((rstruct = (RadioStruct) radioMenus.get(new Integer(type))) == null){
      rstruct = new RadioStruct(type);
      radioMenus.put(new Integer(type), rstruct);
      if (PluginTypes.isControler(type)) controlerMenu.add(rstruct.getMenu());
      else if (PluginTypes.isView(type)) viewMenu.add(rstruct.getMenu());
    }
    rstruct.addRadioButton(name, cl);
  }

  /**
   * Instantiate the class and try to get the type of the plugin
   * @param cl the class
   * @return the type of the plugin
   * @see fr.unice.javinspector.plugin.PluginTypes#
   */
  private int getType(Class cl){
    int res = -1;
    try {
      Plugin pl = (Plugin) cl.newInstance();
      res = pl.getType();
    }
    catch (IllegalAccessException ex) {
      JOptionPane.showMessageDialog(this.getTopLevelAncestor(),
                                    "Failed to load one of the plugins",
                                    "IllegalAccessException",
                                    JOptionPane.ERROR_MESSAGE);
    }
    catch (InstantiationException ex) {
      JOptionPane.showMessageDialog(this.getTopLevelAncestor(),
                                    "Failed to load one of the plugins",
                                    "InstantiationException",
                                    JOptionPane.ERROR_MESSAGE);
    }
    catch (ClassCastException ex) {
      JOptionPane.showMessageDialog(this.getTopLevelAncestor(),
                                    "The plugin Does not implement " + Plugin.class.getName(),
                                    "ClassCastException",
                                    JOptionPane.ERROR_MESSAGE);
    }
    catch (Exception e) {
      JOptionPane.showMessageDialog(this.getTopLevelAncestor(),
                                    "Unknown error : " + e.getMessage(),
                                    e.getClass().getName(),
                                    JOptionPane.ERROR_MESSAGE);
    }
    return res;
  }

  public AbstractTopManager updateSelectedPlugins(AbstractTopManager man){
    Iterator iter = radioMenus.keySet().iterator();
    while (iter.hasNext()) {
      Integer item = (Integer)iter.next();
      man.setPlugin(item,
                    ((RadioStruct) radioMenus.get(item)).getSelectedClass());
    }
    return man;
  }

  public PluginSettings getPluginSettings(){
    PluginSettings res = new PluginSettings();
    Iterator iter = radioMenus.keySet().iterator();
    while (iter.hasNext()) {
      Integer item = (Integer)iter.next();
      res.put(item, ((RadioStruct) radioMenus.get(item)).getSelectedClass());
    }
    return res;
  }

}


class RadioStruct implements ActionListener{
  private ButtonGroup bGroup;
  private JMenu menuContainer;
  private Class selectedClass;
  private int pluginType;
  private Hashtable clazzs;

  public RadioStruct(int type){
    bGroup = new ButtonGroup();
    menuContainer = new JMenu(PluginTypes.getName(type));
    selectedClass = null;
    pluginType = type;
    clazzs = new Hashtable();
  }

  public JMenu getMenu(){
    return menuContainer;
  }


  public void addRadioButton(String buttonText, Class clazz){
    JRadioButtonMenuItem leRB = new JRadioButtonMenuItem(buttonText);
    bGroup.add(leRB);
    menuContainer.add(leRB);
    leRB.addActionListener(this);
    leRB.setActionCommand(buttonText);
    if (menuContainer.getItemCount() == 1) {
      leRB.setSelected(true);
      selectedClass = clazz;
    }
    clazzs.put(buttonText, clazz);
  }

  public void actionPerformed(ActionEvent e){
      selectedClass = (Class)clazzs.get(e.getActionCommand());
  }

  public Class getSelectedClass(){
    return selectedClass;
  }
}
