package fr.unice.javinspector.GUI.main;

import java.io.*;
import java.awt.*;
import javax.swing.*;
import org.w3c.dom.*;
import org.xml.sax.*;
import java.awt.event.*;
import javax.xml.parsers.*;

/**
 * TipsFrame is like a generic composant which displays some tips at the begining
 * of a logiciel.<br>It reads a xml file and display into a frame the tips written in xml.
 * You can specify the name of the xml file and the tag where are insered the tips.
 * <br>By default the name of the file is "bdTip.xml" and the tag is "tip".
 * This is an example of a xml file of tips.<br>
 * <pre>
 *  &lt; bd &gt; 
 *	 &lt; tips &gt; 
 *		&lt; tip &gt; tip1 here
 *		&lt; /tip &gt; 
 *		&lt; tip &gt; tip2 here
 *		&lt; /tip &gt; 
 *		...
 *      &lt; /tips &gt; 
 *  &lt; bd &gt; 
 * <pre>
 * <br>
 * TipsFrame erases all spaces , blanks and end of line and so can generate a well display
 * in function of the size of screen.
 * <br> There are some buttons in order to display next or prev tips. You can specify too if
 * you want to display the tips of the day at each start of the application using a boolean.
 * <br>
 * There is also an animation on the left of the frame.
 * <br> NB :the xml file can be placed into a jar.
 * @version final 09/03/04 [ modified 12/05/04 ]
 * @author ARNAUD J-M
 * @see AnimatedPanel
 */
public class TipsFrame extends JFrame {

	/** This String eol represents the end of line */
	public final static String eol = System.getProperty("line.separator");
	public final static String fSep = System.getProperty("file.separator"); // file separator


	/** the number of character displayed by line in the main JTextArea */
	private static int NBRECOLONES; // will be filled in void initProp()
	/** the main pane of JFrame*/
	private Container pane;
	/** The text of the current tip */
	private JTextArea jtxt = new JTextArea();
	/** the index of current tip */
	private JTextArea jnum = new JTextArea();
	/** the NodeList of node tag */
	private NodeList list;
	/** the number of tip in the xml file */
	private int nbreTips;
	/** the index of current tip */
	private int index; 
	/** The animatedPanel which animates an ampoule, 11 gif,  cycle, 50 ms, w=85, h=150*/
	private AnimatedPanel animPanel= new AnimatedPanel("imgs/animAmpoule/ampoule",11,85,150/*,true,50*/);

	/** the JRadioButton indicate if we want to display tip of the day at each time we launch the application*/
	private JRadioButton still = new JRadioButton("show tips on startup",true);
	
	/** 
	 * This constructor without parameter , displays the tips of the day for a xml file
	 * called "bdTip.xml" with tag "tip".
	 */
	public TipsFrame(){
		this("bdTip.xml","tip");
		}
	
	/** 
	 * This constructor displays the tips of the day for a xml file
	 * called the string in parameter with tag "tip".
	 * @parameter fichierXml the url of the xml file.
	 */
	public TipsFrame(String fichierXml){
		this(fichierXml,"tip");
		}
	
	/** 
	 *This constructor displays the tips of the day for a xml file
	 * called the string in parameter with tag string in parameter.
	 * @parameter fichierXml the url of the xml file.
	 * @parameter tag the nameof the tag.
	 */
	public TipsFrame(String fichierXml , String tag){
		super("Tip of the day");
		
		list = getNodeList(fichierXml,tag);
		nbreTips = list.getLength();
		index = rand(0,nbreTips);
		
		//the properties of the JTextArea
		jtxt.setEditable(false);
    	jnum.setFont(new Font(null,Font.ITALIC,12));
    	jnum.setBackground(Color.lightGray );
        jnum.setEditable(false);
        
		initProp();
		initPanel();
		
	    displayTip(index);
		animPanel.startAnimation();
		}
	
	/**
	 * This procedure initializes buttons and panels.
	 */	
	private void initPanel(){
   pane = getContentPane();
  
  // definition of the button close
  JButton close = new JButton("close");
  close.setToolTipText("close tip");
  //close.setActionCommand("bouton");
  close.setMnemonic(KeyEvent.VK_C);
  close.addActionListener(new ActionListener() {
	public void actionPerformed(ActionEvent e) {
      animPanel.stopAnimation();
	  dispose();
	}});
      
  // definition of the button next
  JButton next = new JButton("next");
  next.setToolTipText("next tip");
  //next.setActionCommand("bouton");
  next.setMnemonic(KeyEvent.VK_N);
  next.addActionListener(new ActionListener() {
	public void actionPerformed(ActionEvent e) {
	  index = (index + 1) % nbreTips;
	  displayTip(index);
	}});
	
	// definition of the button next
  JButton prev = new JButton("prev");
  prev.setToolTipText("previous tip");
  prev.setActionCommand("bouton");
  prev.setMnemonic(KeyEvent.VK_P);
  prev.addActionListener(new ActionListener() {
	public void actionPerformed(ActionEvent e) {
		if(index==0) index+=nbreTips;
	  index--;
	  displayTip(index);
	}});

  /******************** east panel contains the button *************/
  JPanel east = new JPanel();
  east.setLayout(new GridLayout(3,1));
  east.add(prev);
  east.add(next);
  east.add(close);
  pane.add(east,BorderLayout.EAST);
  
  /******************** south panel contains the JRadiobutton *************/
   JPanel south = new JPanel();
   
   south.add(still);
   pane.add(south,BorderLayout.SOUTH);
  
  /******************** jscp contains the text of the current tip ******/
  	//JPanel tip = new JPanel();
 	//tip.setBackground(Color.white);
 	//tip.add(jtxt);
 	//pane.add(tip,BorderLayout.CENTER);
 	/* for the scroll */
 	JScrollPane jscp = new JScrollPane(jtxt);
 	pane.add(jscp,BorderLayout.CENTER);
 	
    
    /******************** north panel contains the JLabel and the numPanel*************/
    JPanel north = new JPanel();
    north.setBackground(Color.lightGray);
    JTextArea know = new JTextArea("    Did you know ...     ");
    know.setBackground(Color.lightGray );
    know.setEditable(false);
    know.setFont(new Font(null,Font.BOLD,20));
    north.add(know,BorderLayout.CENTER);
    
    /******************** num panel contains the index of current tip *************/
    JPanel numPanel = new JPanel();
    numPanel.setBackground(Color.lightGray );
    numPanel.add(jnum);
    north.add(numPanel,BorderLayout.EAST);
    
    pane.add(north,BorderLayout.NORTH);
    
    /******************** west panel contains the animation *************/
    JPanel west = new JPanel();
    west.setBackground(Color.white);
    west.add(animPanel,BorderLayout.CENTER);
    pane.add(west,BorderLayout.WEST);
	}
	
 
 /** 
  * This procedure initializes the properties of the JFrame.
  */	 	
private void initProp(){
	setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
    addWindowListener(new WindowAdapter() {
        public void windowClosing(WindowEvent e) {
	       animPanel.stopAnimation();
	       dispose();
	}						   
      });
	Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();   
    setLocation(screenSize.width / 4,screenSize.height / 3);
    setSize(screenSize.width / 2,screenSize.height / 3);
    setResizable(false);
    NBRECOLONES=screenSize.width/24;
}

/**
 * displayTip writes the tip corresponding to tip i in the file xml.
 * @param i the index of tip to display.
 */
private void displayTip(int i){
	writeText(format(getTipString(i)));
}

/**
 * writeText writes the text in param and the number of tip displayed.
 * it repaint the JFrame.
 */ 
private void writeText(String text){
  	jtxt.setText(eol+eol+" "+text); 
    jnum.setText("    [num "+(index+1)+"/"+nbreTips+"]");
    pane.validate();
    repaint();
}

/** To get the text of the tip in the file xml.
 * @param i the index of tip to get.
 * @return the String txt of the tip number i.
 */
private String getTipString(int i){
	return list.item(i).getFirstChild().getNodeValue();
}

/**
 * trimAll erase EOL and blank character replacing them by a space.
 * @param chaine the String to simplify.
 * @return the StringBuffer simplified.
 */
private StringBuffer trimAll(String chaine){
	StringBuffer res = new StringBuffer(chaine.trim().replaceAll("[ \t\n\f\r]+"," "));
	return res;
}

/** 
 * This function will format a String before display it graphically.
 * @param chaine the String to format.
 * @param the String formated.
 */
private String format(String chaine){
	StringBuffer res = trimAll(chaine);
	//attention pb si un mot = nbrecolone
	for(int i=NBRECOLONES;i<res.length();i+=NBRECOLONES){
	/*	int	stop = i-NBRECOLONES+2;
		System.out.println(i);*/
		while(res.charAt(i)!=' '){
			i--;
		/*	if (i==stop){
				i+=NBRECOLONES-2; 
				break;
				}*/
			}
		res.insert(i,eol+" ");
	}
	return res.toString();
}

/**
 * Tis function parses a xml file and returns the NodeList of tag.
 * @param fichierXml the String the xml file to parse.
 * @param tag  the String of the tag.
 * @return the NodeList of nodes corresponding to tag.
 */
private NodeList getNodeList(String fichierXml, String tag){
	Document document = null;
    DocumentBuilderFactory factory = null;

    try {
      factory = DocumentBuilderFactory.newInstance();
      DocumentBuilder builder = factory.newDocumentBuilder();
      document = builder.parse(TipsFrame.class.getResourceAsStream("/"+fichierXml));
    } catch(IOException ioe) {
    	System.err.println("[TipsFrame.java] input/ouput error");ioe.printStackTrace();
    }
    catch(SAXException sax) {
    	System.err.println("[TipsFrame.java] parse error : tips.xml has been modified");
    	sax.printStackTrace();
    }
    catch(IllegalArgumentException nul) {
    	System.err.println("[TipsFrame.java] tips.xml has not been found : "+fichierXml);
    	nul.printStackTrace();
    }
    	catch (Exception e) {
    	System.err.println("[TipsFrame.java] IMPOSSIBLE BUG SYSTEM !!!!");
       e.printStackTrace();
    }    
    return document.getElementsByTagName(tag);	
}

/**
 * This function will tell if the user want to disaplay tip of the day on startup
 * @return true for displaying on startup , false ohterwise.
 */
public boolean getBooleanShowTipOnStartup(){
	return still.isSelected();
}


/**
 * A basic function returning a random int.
 * @param low the minimum value of the random we want.
 * @param high the maximum value of the random we want.
 * @return an int random between low and high.
 */
static int rand(int low, int high) {
    return (int) ( (double) low + Math.random() * (double) (high - low));
  }

/**
 * To test the Frame.
 */
public static void main (String [] args){
	TipsFrame essai = new TipsFrame("test.xml");
	essai.show(); 
}
	
}
