package fr.unice.javinspector.GUI.util;

import javax.swing.JTextPane;
//import fr.unice.javinspector.util.DictionnaryWordStyle;
import java.util.TimerTask;
import java.util.Timer;
import java.util.Hashtable;
import javax.swing.text.Style;
import javax.swing.text.StyleContext;
import javax.swing.text.StyleConstants;
import java.awt.Color;
import javax.swing.text.Document;
import javax.swing.text.*;
import java.util.StringTokenizer;
import javax.swing.BorderFactory;
import javax.swing.border.*;

/**
 * <p>Title: JTextPaneAutoStyled </p>
 * <p>Description: This class is a special class that extends JTextPane.
 * The services that are addon is a text verification with a dictionnary.
 * This allow to put some style on some word( AND OR ..).
 * This class use a timer thread that check do a lexical analys on the text in
 * the JTextPane. </p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: UNICE </p>
 * @author Zerbib Gabriel - Charbit Lucas - Galbois Xavier - Gahide Jrme
 * @version 1.0
 * @see JTextPane
 */

public class JTextPaneAutoStyled extends JTextPane{

  Hashtable dictionnaryWordStyle;
  private Border border1;
  private TitledBorder titledBorder1;

  /**
   * Construct a new JTextPane with the dictionnary. It's also initialize thz timer
   * for checking the word.
   * @param dictionnaryWordStyle the dictionnary to use for the style affectation for a word.
   */
  public JTextPaneAutoStyled(Hashtable dictionnaryWordStyle) {
    super();
    this.dictionnaryWordStyle = dictionnaryWordStyle;

    this.setDoubleBuffered(false);


    new FrameTimer(this , 1000);
    initStylesForTextPane(this);
  }


  /**
   * Check if the style in the dictionnary exist in this JTextPane.
   * If not, a default style will replace the non exist style.
   */
  private void checkStyleName(){

  }


  /**
   * Add some Style to the textPane passed in parameters.
   * @param textPane the textPane to init
   */
  protected void initStylesForTextPane(JTextPane textPane) {

    //Initialize some styles.
    Style def = StyleContext.getDefaultStyleContext().
        getStyle(StyleContext.DEFAULT_STYLE);

    Style regular = textPane.addStyle("regular", def);
    StyleConstants.setFontFamily(def, "SansSerif");
    //StyleConstants.setForeground(def, new Color(0,0,0));
    StyleConstants.setForeground(def, new Color(255,255,255));

    // a criterion
    Style s = textPane.addStyle("criterion", regular);
    StyleConstants.setBold(s, true);
    StyleConstants.setForeground(s, new Color(0,0,140));

    // a AND OR ..
    s = textPane.addStyle("LinkWord", regular);
    StyleConstants.setItalic(s, true);
    StyleConstants.setFontSize(s,12);
    StyleConstants.setForeground(s,Color.RED);

    // parenthese
    s = textPane.addStyle("parenthese", regular);
    StyleConstants.setBold(s, true);
    StyleConstants.setForeground(s,Color.GREEN);

    // Error
    s = textPane.addStyle("Error", regular);
    StyleConstants.setBold(s, false);
    StyleConstants.setUnderline(s,true);
    StyleConstants. setFontFamily(s,"Verdana");
    //StyleConstants.setForeground(s, new Color(0,0,0));
    StyleConstants.setForeground(def, new Color(255,255,255));
  }

  /**
   * This reStyle all the words of the JTextPane with the dictionnary and
   * the styles that the JTextPane knows.
   * this procedure is space sensitive. So for exemple, if a word AND & OR
   * can be detected, ANDOR will not be .
   */
  public void reStyleWords(){
    Document doc = this.getDocument();
    String text = this.getText();
    this.setText("");

    StringTokenizer st = new StringTokenizer(text," ");

    this.setBackground(Color.LIGHT_GRAY);

    while( st.hasMoreTokens()){
      String token = st.nextToken();

      try {
        if( token.compareTo(" ") == 0){}

        else if( token.compareTo("AND") == 0 ||
                 token.compareTo("and") == 0 ||
                 token.compareTo("&") == 0   ||
                 token.compareTo("OR") == 0  ||
                 token.compareTo("or") == 0  ||
                 token.compareTo("|") == 0   ||
                 token.compareTo("NOT") == 0 ||
                 token.compareTo("!") == 0 ){
          String style = (String)dictionnaryWordStyle.get(token);
          if( style == null )
            style = "LinkWord";
          doc.insertString(doc.getLength(), token+" ", this.getStyle(style));
        }
        else if( token.compareTo("(") == 0 ||
                 token.compareTo(")") == 0 ){
          String style = (String)dictionnaryWordStyle.get(token);
          if( style == null )
            style = "parenthese";
          doc.insertString(doc.getLength(), token+" ", this.getStyle(style));
        }
        else if( ((String)dictionnaryWordStyle.get(token)) != null){
          String style = (String) dictionnaryWordStyle.get(token);
          doc.insertString(doc.getLength(), token+" ", this.getStyle(style));
        }
        else{ // the last action possible : an error
          doc.insertString(doc.getLength(), token, this.getStyle("Error"));
          doc.insertString(doc.getLength(), " ", this.getStyle("regular"));
        }

      }
      catch (BadLocationException ex) {
        ex.printStackTrace();
      }

    }
    //this.updateUI();
    //this.repaint();
    //this.paintAll(this.getGraphics());

  }

  public JTextPaneAutoStyled() {
    try {
      jbInit();
    }
    catch(Exception e) {
      e.printStackTrace();
    }
  }

  private void jbInit() throws Exception {
    border1 = BorderFactory.createBevelBorder(BevelBorder.LOWERED,Color.white,Color.white,new Color(103, 101, 98),new Color(148, 145, 140));
    titledBorder1 = new TitledBorder(border1,"Enter your Query");
    this.setBorder(titledBorder1);
  }

}

/**
 *
 * <p>Title: FrameTimer </p>
 * <p>Description: This inner class provide a timer to the
 * JTextPaneAutoStyled. Every times the timer activate himself,
 * the JTextPaneAutoStyled check his word and style them.</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: </p>
 * @author Zerbib Gabriel - Charbit Lucas - Galbois Xavier - Gahide Jrme
 * @version 1.0
 * @see JTextPaneAutoStyled#reStyleWords() reStyleWords
 */
class FrameTimer extends TimerTask{

  private Timer timer;
  private JTextPaneAutoStyled textPane;

  /**
   * Construct a new timer that every timeTorefresh will lunch
   * the lexical analys on the textPane
   * @param textPane the textPane the monitoring
   * @param timeTorefresh the delay to munch the verfication.
   */
  public FrameTimer(JTextPaneAutoStyled textPane, int timeTorefresh){
    this.textPane= textPane;
    timer = new Timer();
    timer.schedule(this,timeTorefresh,timeTorefresh);
  }

  /**
   * the run method like for every thread.
   */
  public void run() {
    //System.out.println("reStyling Words of textPane");
    textPane.reStyleWords();
  }

}

