package fr.unice.javinspector.MVC.controler;

import fr.unice.javinspector.MVC.view.View;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import fr.unice.javinspector.MVC.manager.Manager;

/**
 * <p>Title: Class AbstractControler</p>
 * <p>Description: This class can be extanded to implement a controler faor a certain view</p>
 * <p>Company: UNSA</p>
 * @author Jerme GAHIDE - Lucas CHARBIT - Gabriel ZERBIB - Xavier GALBOIS
 * @version 1.0
 */

public abstract class AbstractControler implements Controler {
  // the manager of this controler
  Manager manager;

  public AbstractControler(){
  }

  /**
   * defaut constructor from a manager
   * @param m the manager
   */
  public AbstractControler(Manager m){
    manager = m;
  }

  /**
   * sets this controler's manager
   * @param m the new manager
   */
  public void setManager(Manager m){
    manager = m;
  }

  /**
   * gets this controler's manager
   * @return this controler's manager
   */
  public Manager getManager(){
    return manager;
  }

  /**
   * Called to instantiate a controler
   * @param theControler the controler Class to instantiate
   * @param man the first parameter (the manager of the controler)
   * @param view the wiew to be controled by the controler
   * @return the new controler instantiated
   */
  public static Controler create(Class theControler, Manager man, View view){
    Controler res = null;
        Class[] viewArgsClass = new Class[] {Manager.class, View.class};
            Object[] viewArgs = new Object[] {man, view};
            Constructor viewArgsConstructor;

            try {
              viewArgsConstructor = theControler.getConstructor(viewArgsClass);
              res = (Controler) createObject(viewArgsConstructor, viewArgs);
          } catch (NoSuchMethodException e) {
              System.err.println(e);
          }
          return res;
  }

  /**
   * Creates an object from a constructor and its arguments
   * @param constructor the constructor to use
   * @param arguments the arguments of the constructor
   * @return the newly instantiated object
   */
  private static Object createObject(Constructor constructor,
                                  Object[] arguments) {

   Object object = null;

   try {
     object = constructor.newInstance(arguments);
     return object;
   } catch (InstantiationException e) {
     e.printStackTrace();
   } catch (IllegalAccessException e) {
     e.printStackTrace();
   } catch (IllegalArgumentException e) {
     e.printStackTrace();
   } catch (InvocationTargetException e) {
     e.getTargetException().printStackTrace();
   }
   return object;
}
}



