package fr.unice.javinspector.MVC.manager;

import fr.unice.javinspector.plugin.*;
import fr.unice.javinspector.MVC.controler.*;
import fr.unice.javinspector.MVC.view.*;
import java.awt.Container;
import fr.unice.javinspector.core.analyser.Inspector;
import fr.unice.javinspector.core.analyser.ClassifierAnalyser;
import java.util.Hashtable;


/**
 * <p>Title: JavInspector</p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: </p>
 * @author Jerme GAHIDE - Lucas CHARBIT - Gabriel ZERBIB - Xavier GALBOIS
 * @version 1.0
 */
public abstract class AbstractTopManager extends AbstractManager {
  private Inspector inspector;
  private Hashtable pluginSettings;

  /**
   * default constructor : build a top manager with an inspector
   * and the plugin settings
   * @param inspector the inspector
   * @param pluginSettings the plugin settings
   */
  public AbstractTopManager(Inspector inspector, Hashtable pluginSettings){
    super(null);
    this.inspector = inspector;
    this.pluginSettings = pluginSettings;
  }

  /**
   * default implementation : returns PluginTypes.TOP_MANAGER
   * @return PluginTypes.TOP_MANAGER
   */
  public int getType(){
    return PluginTypes.TOP_MANAGER;
  }

  /**
   * return the inspector
   * @return the inspector
   */
  public Inspector getInspector(){
    return inspector;
  }

  /**
   * sets the plugin for the given type
   * @param type the type
   * @param clazz the Class of the plugin
   */
  public void setPlugin(int type, Class clazz){
    pluginSettings.put(new Integer(type), clazz);
  }

  /**
   * sets the plugin for the given type
   * @param type the type
   * @param clazz the Class of the plugin
   */
  public void setPlugin(Integer type, Class clazz){
    pluginSettings.put(type, clazz);
  }

  /**
   * gets the plugin for the given type
   * @param type the type
   * @return the class plugin
   */
  public Class getPlugin(int type) {
    Class res = (Class) pluginSettings.get(new Integer(type));
    if (res == null) throw new NullPointerException();
    return res;
  }

  public abstract void addHeritageView(ClassifierAnalyser ca);
  public abstract void addDetailView(ClassifierAnalyser ca);
  public abstract void addCompositionView(ClassifierAnalyser ca);
  public abstract void addSearchView(ClassifierAnalyser ca);

  /**
   * get the top manager (this)
   * @return <code>this</code>
   */
  public AbstractTopManager getTopManager() {
    return this;
  }
}
