package fr.unice.javinspector.MVC.manager;


import fr.unice.javinspector.plugin.*;
import fr.unice.javinspector.MVC.controler.*;
import fr.unice.javinspector.MVC.view.*;
import java.awt.*;
import javax.swing.*;
import fr.unice.javinspector.GUI.util.*;
import fr.unice.javinspector.core.analyser.ClassifierAnalyser;
import java.util.Hashtable;
import fr.unice.javinspector.core.analyser.Inspector;
import java.util.Enumeration;
import java.awt.event.ActionListener;
import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;


/**
 * <p>Title: Class JTabbedPaneHeritageManager</p>
 * <p>Description: This class implements AbstractManager. This manager show his views in a
 * JTabbedPane.</p>
 * <p>Company: UNSA</p>
 * @author Charbit Lucas - Gahide Jrome - Galbois Xavier - Zerbib Gabriel
 * @version 1.0
 */
public class TabbedPaneHeritageManager extends AbstractManager{
  // to count the panels of the tabbedPane
  private int index = 0;
  // the icons for the tabbedPane
  private Icon classTreeIcon = IconBank.getClassIcon();
  private Icon interfaceTreeIcon = IconBank.getInterfaceIcon();
  private Icon internalClassIcon = new ImageIcon(TabbedPaneHeritageManager.class.getResource("/imgs/internalclass.gif"));
  private Icon internalInterfaceIcon = new ImageIcon(TabbedPaneHeritageManager.class.getResource("/imgs/internalinterface.gif"));
  private JPanel globalPanel = new JPanel();
  private BorderLayout borderLayout1 = new BorderLayout();
  private JPanel jPanelNbViews = new JPanel();
  private FlowLayout flowLayout1 = new FlowLayout();
  private JTabbedPaneWithCloseIcons jTabbedPaneHeritageView = new JTabbedPaneWithCloseIcons(false);
  private JLabel jLabelDescription = new JLabel();

  /**
   * The constructor.
   * @param topManager : the father manager of this manager, very important to allow delegation.
   */
  public TabbedPaneHeritageManager(Manager topManager) {
    super(topManager);
    try {
      jbInit();
    }
    catch (Exception e) {
      e.printStackTrace();
    }
    // we create the first panel in the JTabbedPane
    this.addHeritageView(this.getInspector().getMainClassifier());
  }

  /**
   * This function comes from the AbstractManager interface.
   * @return : a string containing the name of this class.
   */
  public String getName() {
    return this.getClass().getName();
  }

  /**
   * This function comes from the View interface.
   * @return : the container where the future graphic objects will be placed.
   */
  public Container getViewContainer() {
    return globalPanel;
  }

  /**
   * This function create a new HeritageView for the selected ClassifierAnalyser.
   * As the view and controler setted by he user are some plugins, we need to get the name
   * of their classes. After, a new view and a new controler are instancied using the create method
   * of the abstract classes.
   * @param ca
   */
  public void addHeritageView(ClassifierAnalyser ca) {
    Class heritageViewClass = this.getPlugin(PluginTypes.HERITAGE_VIEW);
    Class heritageControlerClass = this.getPlugin(PluginTypes.HERITAGE_CONTROLER);
    HeritageView hv = null;
    HeritageControler hc = null;
    try {
      hv = (HeritageView)AbstractView.create(heritageViewClass, ca);
      hc = (HeritageControler)AbstractControler.create(heritageControlerClass, this, hv);
    }
    catch(Exception e) {
      e.printStackTrace();
      JOptionPane.showMessageDialog(this.getViewContainer(), "Can't create view",
                                    "Heritage view creation error message", JOptionPane.ERROR_MESSAGE);
      return;
    }
    // controlersViews.put(hc, hv);
    String classifierName = ca.getClassAnalysed().getName();
    String shortClassifiername = ClassifierAnalyser.shortName(classifierName);
    String modifier = ca.modifierToString();
    // the last parameter is the tooltip text for the panel.
    if(ca.getClassAnalysed().isInterface()) {
      jTabbedPaneHeritageView.addTab(shortClassifiername, hv.getViewContainer(),
                                     interfaceTreeIcon, modifier + " " + classifierName);
      jTabbedPaneHeritageView.setSelectedIndex(jTabbedPaneHeritageView.getTabCount() - 1);
    }
    else {
      jTabbedPaneHeritageView.addTab(shortClassifiername, hv.getViewContainer(),
                                     classTreeIcon, modifier + " Class " + classifierName);
      jTabbedPaneHeritageView.setSelectedIndex(jTabbedPaneHeritageView.getTabCount() - 1);
    }
  }

  /**
   * GUI initilisation.
   * @throws Exception
   */
  private void jbInit() throws Exception {
    globalPanel.setLayout(borderLayout1);
    jPanelNbViews.setLayout(flowLayout1);
    flowLayout1.setAlignment(FlowLayout.CENTER);
    flowLayout1.setVgap(0);
    jTabbedPaneHeritageView.setBackground(new Color(196, 230, 154));
    jTabbedPaneHeritageView.setMaximumSize(new Dimension(900, 900));
    jTabbedPaneHeritageView.setMinimumSize(new Dimension(0, 0));
    globalPanel.setBackground(SystemColor.control);
    jPanelNbViews.setBorder(BorderFactory.createLoweredBevelBorder());
    jPanelNbViews.setMinimumSize(new Dimension(0, 0));
    jPanelNbViews.setToolTipText("");
    jLabelDescription.setBorder(null);
    jLabelDescription.setMinimumSize(new Dimension(0, 0));
    jLabelDescription.setIconTextGap(0);
    jLabelDescription.setText("Heritage views manager");
    globalPanel.add(jPanelNbViews, BorderLayout.NORTH);
    jPanelNbViews.add(jLabelDescription, null);
    globalPanel.add(jTabbedPaneHeritageView, BorderLayout.CENTER);
  }
}
