package fr.unice.javinspector.core.XML;

import java.io.*;
import java.util.*;
import javax.swing.*;
import fr.unice.javinspector.core.analyser.*;

/**
 * <p>Title: class SaveFileXML</p>
 * <p>Description: this class allows to save the detail view in XML format</p>
 * <p>Company: UNSA</p>
 * @author Lucas Charbit - Jerome Gahide - Galbois Xavier - Gabriel Zerbib
 * @version 1.0
 */
public class SaveFileXML {

  private static String newline = System.getProperty("line.separator");

  /**
   * this function write the contents of list in bfwriter if bool equals
   * true
   * @param bfwriter the BufferedWriter where we want to write
   * @param list contains all the String we want to write
   * @param bool the boolean who says if we must write or no
   */
  public static void writeElements(BufferedWriter bfwriter, ArrayList list,
                                   boolean bool) {
    try {
      if (bool) {
        for (Iterator i = list.iterator(); i.hasNext(); ) {
          bfwriter.write( ((String) i.next()).trim());
          bfwriter.write(System.getProperty("line.separator"));
        }
      }
      else {
        bfwriter.write("EMPTY");
        bfwriter.write(System.getProperty("line.separator"));
      }
    }
    catch (Exception e) {


    }
  }
  /**
   * this function saves the Classifier cl in the File file
   * @param file the File where we want to write the classifier
   * @param cl the classifier we want to write
   */
  public static void save(File file, ClassifierAnalyser cl) {

    String name = cl.toString();
    ArrayList listName = new ArrayList();
    listName.add(name);
    if(cl.getClassAnalysed().getPackage() != null)
      listName.add(cl.getClassAnalysed().getPackage().toString());

    ConstructorAnalyser[] cons = cl.getConstructor();
    ArrayList constructorList = new ArrayList();
    for (int i = 0; i < cons.length; i++) {
      constructorList.add(cons[i].getFullName());
    }

    FieldAnalyser[] field = cl.getField();
    ArrayList fieldList = new ArrayList();
    ArrayList fieldStaticList = new ArrayList();
    for (int i = 0; i < field.length; i++) {
      if (!field[i].isStatic())
        fieldList.add(field[i].getFullName());
      else
        fieldStaticList.add(field[i].getFullName());
    }

    MethodAnalyser[] method = cl.getMethod();
    ArrayList methodList = new ArrayList();
    ArrayList methodStaticList = new ArrayList();
    ArrayList methodGet = new ArrayList();
    ArrayList methodSet = new ArrayList();
    for (int i = 0; i < method.length; i++) {
      if (method[i].isStatic())
        methodStaticList.add(method[i].getFullName());
      else if(method[i].isGetter()){
        methodGet.add(method[i].getFullName());
      }
      else if(method[i].isSetter()){
        methodSet.add(method[i].getFullName());
      }
      else {
        methodList.add(method[i].getFullName());
      }
    }

    Class[] classes = cl.getInternalClasses();
    ArrayList tmp =  cl.getAnonymousInternalClasses();
    ArrayList classList = new ArrayList();
    for (int i = 0; i < classes.length; i++) {
      classList.add(ClassifierAnalyser.modifierToString(classes[i]) + " " +
                    ClassifierAnalyser.nameInternalClass(classes[i]));
    }

    for (int i = 0; i < tmp.size(); i++) {
      classList.add(ClassifierAnalyser.modifierToString((Class)tmp.get(i)) + " " +
                    ((Class)tmp.get(i)).getName());
    }

    JFileChooser fileChooser = new JFileChooser();
    fileChooser.setFileFilter(new XMLFilter());
    fileChooser.setAcceptAllFileFilterUsed(false);
    fileChooser.setDialogType(JFileChooser.SAVE_DIALOG);

    int choix = 12345;
    int act = JOptionPane.NO_OPTION;
    while (act == JOptionPane.NO_OPTION) {
      choix = fileChooser.showSaveDialog(null);
      if (choix == fileChooser.APPROVE_OPTION) {
        file = fileChooser.getSelectedFile();
        if (!file.getName().endsWith(".xml")) {
          file = new File(file.getAbsolutePath() + ".xml");
        }
        if (file.exists()) {
          act = JOptionPane.showConfirmDialog(null, file.toString() +
                                              " exists.  Overwrite?",
                                              "Save the File",
                                              JOptionPane.YES_NO_OPTION);
        }
        else
          break;
      }
      else
        break;
      if (act == JOptionPane.YES_OPTION) {
        break;
      }
    }
    if(choix == JFileChooser.CANCEL_OPTION){
      return;
    }
    if(file == null){
      return;
    }
    try {
      BufferedWriter bfwriter = new BufferedWriter(new FileWriter(file));
      bfwriter.write("<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>" +
                     newline +
                     newline +
                     "<document>" + newline);


      bfwriter.write("<class-name>" + newline);
      writeElements(bfwriter, listName, true);
      bfwriter.write("</class-name>" + newline);

      bfwriter.write("<instance-fields>" + newline);
      writeElements(bfwriter, fieldList, true);
      bfwriter.write("</instance-fields>" + newline);

      bfwriter.write("<static-fields>" + newline);
      writeElements(bfwriter, fieldStaticList, true);
      bfwriter.write("</static-fields>" + newline);

      bfwriter.write("<constructors>" + newline);
      writeElements(bfwriter, constructorList, true);
      bfwriter.write("</constructors>" + newline);

      bfwriter.write("<instance-methods>" + newline);
      writeElements(bfwriter, methodList, true);
      bfwriter.write("</instance-methods>" + newline);

      bfwriter.write("<static-methods>" + newline);
      writeElements(bfwriter, methodStaticList, true);
      bfwriter.write("</static-methods>" + newline);

      bfwriter.write("<getter-methods>" + newline);
      writeElements(bfwriter, methodGet, true);
      bfwriter.write("</getter-methods>" + newline);

      bfwriter.write("<setter-methods>" + newline);
      writeElements(bfwriter, methodSet, true);
      bfwriter.write("</setter-methods>" + newline);

      bfwriter.write("<internal-classes>" + newline);
      writeElements(bfwriter, classList, true);
      bfwriter.write("</internal-classes>" + newline);

      bfwriter.write("</document>");

      bfwriter.close();
    }
    catch (Exception e) {

    }
  }

}
