package fr.unice.javinspector.core.analyser;

import java.lang.reflect.Constructor;
import java.util.ArrayList;
import java.lang.reflect.Modifier;
/**
 * <p>Title: ConstructorAnalyser </p>
 * <p>Description: This class represents in fact a constructor and the list of 
 * TypeClass of its parameters : we encapsule this data in this simple class in order 
 * to facilitate the analyse of Constructors.</p>
 * <p>Company: UNSA </p>
 * @author APPIETTO Christophe && ARNAUD Jean-Michel && EL MAHRATI Hassan [ prev : Zerbib Gabriel - Charbit Lucas - Galbois Xavier - Gahide Jrme ]
 * @version 1.0
 * @since j2sdk1.4
 */
public class ConstructorAnalyser{
	/** the Constructor */
  //private Constructor constructor;
  /** an array of TypeClass , for the constructor's parameters */
 // private TypeClass [] listTypeClass;
private Constructor constructor;
  private TypeClass [] listTypeClass;
  private int modifier;
  private ClassifierAnalyser classifierAnalyser;
  /**
   * Construct a new ConstructorAnalyser
   * @param constructor the constructor to analyse
   * @param cl the classifier analyser of this.
   */
  public ConstructorAnalyser(Constructor constructor,ClassifierAnalyser cl) {
    this.constructor = constructor;
    Class [] arrayParameter = constructor.getParameterTypes();
    listTypeClass = new TypeClass[arrayParameter.length];
    for (int i = 0; i < arrayParameter.length; i++)
      listTypeClass[i] = cl.formatParametre(arrayParameter[i]);
      modifier = constructor.getModifiers();
  }

 /**
   * To get the constructor.
   * @return the constructor analysed
   */
  public Constructor getConstructor() {
    return constructor;
  }

  /**
   * To get the array of TypeClass of constructor's parameters.
   * @return TypeClass[]
   */
  public TypeClass[] getListTypeClass() {
    return listTypeClass;
  }
  
  /**
   * To get the string representation of the modifier of the constructor.
   * @return a string representation of the modifier of the constructor
   */
  public String getStringModifier(){
    return Modifier.toString(modifier);
  }

  /**
   * This function returns the full nameof the constructor and the nature of its param. 
   * @return the full name of the constructor.
   */
  public String getFullName(){
    StringBuffer param=new StringBuffer("( ");
    StringBuffer exception = new StringBuffer(" throws ");
    StringBuffer res = new StringBuffer(getStringModifier()+ " " + constructor.getName());
    if(listTypeClass.length>0){
      for (int i = 0; i < listTypeClass.length-1; i++) 
        param.append(listTypeClass[i].getName()+", ");
      param.append(listTypeClass[listTypeClass.length-1].getName());
    }
    param.append(") ");
    res.append(param);
    
    Class [] tmp = constructor.getExceptionTypes();
    if(tmp.length>0){
      for (int j = 0; j < tmp.length-1; j++) 
          exception.append(tmp[j].getName() + ", ");
      exception.append(tmp[tmp.length-1].getName());
      res.append(exception);
      }
    return res.toString();
  }
  
 
  /**
   * The override of toString. Do the same thing of getFullName().
   * @see #getFullName
   * @return the string representation of the analyser
   */
  public String ToString(){
  	return getFullName();
  }

 

}