package fr.unice.javinspector.core.analyser;

import java.util.ArrayList;
import java.lang.reflect.*;
import fr.unice.javinspector.GUI.main.*;
import fr.unice.javinspector.core.loader.*;
import java.util.Hashtable;
import java.util.Iterator;

/**
 *
 * <p>Title: Class Inspector</p>
 * <p>Description: The inspector grab the classifier to study an create an AdvancedAnalyser
 * with it, after, it control the positionment on the hierachy an use tree of the classifier.</p>
 * @version 1.1
 */
public class Inspector {
  /* the starting class name */
  private String classifierName;
  /* the starting class */
  private Class classInspected;
  /* the Starting Classifier Analyzer */
  private ClassifierAnalyser mainClassifierAnalyser;
  /* the class loader */
  private GeneralClassLoader mcl;
  /* the analysers for the differents classes we're going to study */
  private ClassLoader mainClassifierClassLoader;
  private ArrayList advancedAnalysers = new ArrayList();
  private Hashtable classifierAnalysers = new Hashtable();

  /**
   * test program
   * @param theclass the clasifier to study
   */
  public Inspector(Class theclass) {

    try {
    // The class we're dealing with
      mainClassifierClassLoader = theclass.getClassLoader();
      if (mainClassifierClassLoader == null)
        mainClassifierClassLoader = ClassLoader.getSystemClassLoader();
      this.classifierName = theclass.getName();
      classInspected = theclass;
      mainClassifierAnalyser = new ClassifierAnalyser(classInspected, this);
      classifierAnalysers.put(classifierName, mainClassifierAnalyser);
      advancedAnalysers.add(mainClassifierAnalyser);
    }
    catch (Throwable t) {
      throw new NoClassDefFoundError(t.getMessage());
    }
  }

  /**
   * @deprecated use Inspector(Class theclass) only for testing purpose
   * @param thename the name of the inspector
   * @see #Inspector(Class theclass)
   */
  public Inspector(String thename) {
     // The class we're dealing with
     GeneralClassLoader mcl = new FileClassLoader("");
     mainClassifierClassLoader = mcl;
     this.classifierName = thename;
      try {
        classInspected = mcl.loadClass(thename);
      }
      catch (ClassNotFoundException ex1) {
        System.err.println("Class not found !");
        return;
      }
      mainClassifierAnalyser = new ClassifierAnalyser(classInspected,this);
      classifierAnalysers.put(classifierName, mainClassifierAnalyser);
      advancedAnalysers.add(mainClassifierAnalyser);
   }


   /**
    *
    * @return the mainClassifierClassLoader
    */
  public ClassLoader getMainClassLoader(){
    return mainClassifierClassLoader;
  }

   /**
    * adds this classifier to the list of the classifiers
    * @param ca the classifier to add
    */
   public void addClassifierAnalyser(ClassifierAnalyser ca) {
     advancedAnalysers.add(ca);
     classifierAnalysers.put(ca.getClassAnalysed().getName(), ca);
   }

   /**
    * removes this classifier from the list of the classifiers
    * @param ca the clasifer to remove
    */
   public void removeClassifierAnalyser(ClassifierAnalyser ca) {
     classifierAnalysers.remove(ca.getClassAnalysed().getName());
     advancedAnalysers.remove(ca);
   }

   /**
    * @deprecated use ter.API.Inspector#removeClassifierAnalyser(ClassifierAnalyser)
    * @return returns the arraylist of classifiers
    * @see #getClassifierAnalyser(String className)
    * @see #addClassifierAnalyser(ClassifierAnalyser ca)
    * @see #removeClassifierAnalyser(ClassifierAnalyser)
    */
   public ArrayList getAdvancedAnalysers() {
     return advancedAnalysers;
   }

   /**
    * returns a ClassidierAnalyser from a dependance of this inspector
    * @param className the full name of the class
    * @return null if the class could not be loaded
    */
   public ClassifierAnalyser getClassifierAnalyser(String className){
     Class theClass;
     ClassifierAnalyser res = (ClassifierAnalyser) classifierAnalysers.get(className);
     if (res != null) return res;
     else {
       try {
         theClass = mainClassifierClassLoader.loadClass(className);
       }
       catch(Exception e){
         System.err.println("Class " + className +" not loaded !");
         return null;
       }
       res = new ClassifierAnalyser(theClass,this);
       classifierAnalysers.put(theClass.getName(), res);
       advancedAnalysers.add(res);
       return res;
     }
   }
   /**
     * returns a ClassidierAnalyser from a dependance of this inspector
     * @param theClass the class class
     * @return null if the class could not be loaded
     */
    public ClassifierAnalyser getClassifierAnalyser(Class theClass){
      ClassifierAnalyser res = (ClassifierAnalyser) classifierAnalysers.get(theClass.getName());
      if (res != null) return res;
      else {
        res = new ClassifierAnalyser(theClass, this);
        classifierAnalysers.put(theClass.getName(), res);
        advancedAnalysers.add(res);
        return res;
      }
    }

   /**
    * returns the Classifier of the class of this inspector
    * @return the Classifier of the class of this inspector
    */
   public ClassifierAnalyser getMainClassifier(){
     return mainClassifierAnalyser;
   }

   /**
    * get an iterator over the list of classifiers analysers
    * @return an iterator over the list of classifiers analysers
    */
   public Iterator getClassifierAnalyserIterator(){
     return classifierAnalysers.values().iterator();
   }

   /**
    * get the name of the main inspected class
    * @return the name of the main inspected class
    */
   public String getClassifierName() {
     return classifierName;
   }

   /**
    * get the main inspected class
    * @return the main inspected class
    */
   public Class getClassInspected() {
     return classInspected;
   }
  public Hashtable getClassifierAnalysers() {
    return classifierAnalysers;
  }
 }
