package fr.unice.javinspector.core.analyser;


import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.util.ArrayList;
/**
 *
 * <p>Title: MethodAnalyser </p>
 * <p>Description: This class allow to analysed a method </p>
 * <p>Company: UNSA</p>
 * @author Lucas Charbit - Jerome Gahide - Galbois Xavier - Gabriel Zerbib
 * @version 1.0
 */
public class MethodAnalyser {
  private Method method;
  private int modifier;
  private TypeClass[] listTypeClass;
  private TypeClass returnType;
  private ArrayList classesWhereThisMethodIsDefined = new ArrayList();
  private ClassifierAnalyser classifierAnalyser;
  /**
   * the contructor of the Class
   * @param m the method to analyse
   * @param cl the ClassifierAnalyser associated to this Class
   */
  public MethodAnalyser(Method m,ClassifierAnalyser cl) {
    this.method = m;
    modifier = m.getModifiers();
    this.classifierAnalyser = cl;
    Class [] arrayParameter = m.getParameterTypes();
    listTypeClass = new TypeClass[arrayParameter.length];
    for (int i = 0; i < arrayParameter.length; i++) {
      listTypeClass[i] = classifierAnalyser.formatParametre(arrayParameter[i]);
    }
    returnType =classifierAnalyser.formatParametre(method.getReturnType());
  }

  /**
   * get the string of modifier
   * @return the string of modifier
   */
  public String getStringModifier(){
    return Modifier.toString(modifier);
  }
  /**
   * true if the method is static false otherwise
   * @return true if the method is static false otherwise
   */
  public boolean isStatic(){
    return Modifier.isStatic(modifier);
  }

  /**
   * true if the method is static false otherwise
   * @return true if the method is static false otherwise
   */
  public boolean isAbstract(){
    return Modifier.isAbstract(modifier);
  }

  /**
   * return the full name of the method
   * @return return the full name of the method
   */
  public String getFullName(){
    String param="";
    String exception = "throws ";
    String return_type = returnType.getName();
    int i = 0;
    if(listTypeClass.length>0){
      for (i = 0; i < listTypeClass.length-1; i++) {
        param +=listTypeClass[i].getName()+", ";
      }
      param+=listTypeClass[i].getName();
    }
    int j= 0;
    Class [] tmp = method.getExceptionTypes();
    if(tmp.length>0){
      for (j = 0; j < tmp.length-1; j++) {
        try{
          exception += tmp[j].getName() + ", ";
        }
        catch(Exception e){System.err.println("ACTUNG ACHTUNG!!!!!"); e.printStackTrace();}
      }
      exception+=tmp[j].getName();
      return getStringModifier()+ " " + return_type + " " +method.getName() + "("+ param + ") " + exception;
    }
    return getStringModifier()+ " " + return_type + " " + method.getName() + "("+ param + ") ";
  }

  /**
   * return true is the method is a getter false otherwise
   * @return true is the method is a getter false otherwise
   */
  public boolean isGetter(){
    return method.getName().startsWith("get");
  }

  /**
   * return true is the method is a setter false otherwise
   * @return true is the method is a setter false otherwise
   */
  public boolean isSetter(){
    return method.getName().startsWith("set");
  }

  public Method getMethod() {
    return method;
  }
  public int getModifier() {
    return modifier;
  }
  public TypeClass[] getListTypeClass() {
    return listTypeClass;
  }
  public TypeClass getReturnType() {
    return returnType;
  }
  public ArrayList getClassesWhereThisMethodIsDefined() {
    return classesWhereThisMethodIsDefined;
  }
}
