package fr.unice.javinspector.core.loader;

import java.io.IOException;

/**
 *
 * <p>Title: JavInspector</p>
 * <p>Description: Custom class ClassLoader to load classes from jar</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Universit de Nice Sophia Antipolis (UNSA)</p>
 * @author Jerme GAHIDE - Lucas CHARBIT - Gabriel ZERBIB - Xavier GALBOIS
 * @version 1.0
 */

public class JARClassLoader extends GeneralClassLoader {
  private JarResources	jarResources;

  /**
   * Default constructor
   * @param jarName the jar archive from which we shouldload classes
   */
  public JARClassLoader (String jarName) throws IOException {
    // Create the JarResource and suck in the .jar file.
    jarResources = new JarResources (jarName);
  }

  /**
   * loads the class <code>className</code> data from a jar
   * @param className full name of the class
   * @return the bytes of the file
   */
  protected byte[] loadClassBytes (String className){
    // Support the MultiClassLoader's class name munging facility.
    className = formatClassName (className);
    monitor("Loading ressource from JAR : " + className);
    // Attempt to get the class data from the JarResource.
    return (jarResources.getResource (className));
  }

  /**
   * changes the current jar archive in which we search classes
   * @param str
   */
  public void setFilePrefix(String str) throws IOException {
    jarResources = new JarResources (str);
  }

}
