package fr.unice.javinspector.core.loader;

import java.net.URL;
import java.net.URLConnection;
import java.io.InputStream;
import java.io.BufferedInputStream;

/**
 *
 * <p>Title: JavInspector</p>
 * <p>Description: Custom class ClassLoader to load classes from an URL</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Universit de Nice Sophia Antipolis (UNSA)</p>
 * @author Jerme GAHIDE - Lucas CHARBIT - Gabriel ZERBIB - Xavier GALBOIS
 * @version 1.0
 */

public class URLClassLoader
    extends GeneralClassLoader {
  private String urlString;

  /**
   * Default constructor
   * @param urlString - the remote class path
   */
  public URLClassLoader(String urlString) {
    this.urlString = urlString;
  }

  /**
   * loads the class <code>className</code> data from an URL
   * @param className full name of the class
   * @return the bytes of the file
   */
  protected byte[] loadClassBytes(String className) {

    int li;
    if ( (li = className.lastIndexOf('/')) >= 0) {
      urlString = className.substring(0, li + 1);
      className = className.substring(li + 1);
    }
    className = formatClassName(className);
    try {
      print(">>> URL : " + urlString + className);
      URL url = new URL(urlString + className);
      URLConnection connection = url.openConnection();
      if (sourceMonitorOn) {
        print("Loading from URL: " + connection.getURL());
      }
      monitor("Content type is: " + connection.getContentType());

      InputStream inputStream = new BufferedInputStream(connection.
          getInputStream());
      int length = connection.getContentLength();
      monitor("InputStream length = " + length); // Failure if -1

      byte[] data = new byte[length];
      monitor(data.toString());
      int readSize = 0;
      while (readSize != length) {
        readSize = inputStream.read(data, readSize, data.length - readSize); // Actual byte transfer
      }
      inputStream.close();
//         if (readSize != length) {
//           monitor("error reading File : " + readSize + " of " + length + " bytes");
//           inputStream.close();
//           return null;
//         }
      return data;

    }
    catch (Exception ex) {
//         print("### URLClassLoader.loadClassBytes() - Exception: " + ex);
      //ex.printStackTrace();
      return null;
    }
    catch (Error e) {
//       print("### URLClassLoader.loadClassBytes() - Error: " + e);
      //e.printStackTrace();
      return null;

    }
  }

  /**
   * Changes the remote classpath
   * @param str the new remote classpath
   */
  public void setFilePrefix(String str) {
    this.urlString = str;
  }
}
