package fr.unice.javinspector.plugin;

import java.util.Hashtable;

/**
 * <p>Title: Class PluginTypes</p>
 * <p>Description: Defines the different types of plugins</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: </p>
 * @author Jerme GAHIDE - Lucas CHARBIT - Gabriel ZERBIB - Xavier GALBOIS
 * @version 1.0
 */
public final class PluginTypes {
  /**
   * For an HeritageView plugin
   */
  public static final int HERITAGE_VIEW =    0;
  /**
   * For an DetailView plugin
   */
  public static final int DETAIL_VIEW =      1;
  /**
   * For a CompositionView plugin
   */
  public static final int COMPOSITION_VIEW = 2;
  /**
   * For a SearchView plugin
   * This has note been implemented yet (the search results are not Views)
   */
  public static final int SEARCH_VIEW =      3;
  /**
   * For an HeritageControler plugin
   */
  public static final int HERITAGE_CONTROLER =    16;
  /**
   * For a DetailControler plugin
   */
  public static final int DETAIL_CONTROLER =    17;
  /**
   * For a CompositionControler plugin
   */
  public static final int COMPOSITION_CONTROLER =    18;
  /**
   * For a SearchManager plugin
   */
  public static final int SEARCH_CONTROLER =    19;
  /**
   *
   */
  public static final int CONTROLER_MANAGER_LIMIT = 31;
  /**
   *
   */
  public static final int MANAGER = 32;
  /**
   *
   */
  public static final int TOP_MANAGER = 33;
  private static final Hashtable typeNames;
  private static final int CONTROLER_VIEW_LIMIT = 15;

  static {
    typeNames = new Hashtable();
    typeNames.put(new Integer(HERITAGE_VIEW), "Heritage View");
    typeNames.put(new Integer(DETAIL_VIEW), "Detail View");
    typeNames.put(new Integer(COMPOSITION_VIEW), "Composition View");
    typeNames.put(new Integer(SEARCH_VIEW), "Search View");
    typeNames.put(new Integer(HERITAGE_CONTROLER), "Heritage Controler");
    typeNames.put(new Integer(DETAIL_CONTROLER), "Detail Controler");
    typeNames.put(new Integer(COMPOSITION_CONTROLER), "Composition Controler");
    typeNames.put(new Integer(SEARCH_CONTROLER), "Search Controler");
  }

  /**
   * test if a plugin's type is  controler
   * @param p the plugin to test
   * @return true if this plugin is a controler, false otherwise
   */
  public static boolean isControler(Plugin p){
    return (p.getType() > CONTROLER_VIEW_LIMIT && p.getType() < CONTROLER_MANAGER_LIMIT);
  }

  /**
   * test if the type given is a controler type
   * @param type the type to test
   * @return true if this type is a controler, false otherwise
   */
  public static boolean isControler(int type){
    return (type > CONTROLER_VIEW_LIMIT && type < CONTROLER_MANAGER_LIMIT);
  }

  /**
   * test if the given plugin's type is iew
   * @param p the plugin to test
   * @return true if the plugin is a view false otherwise
   */
  public static boolean isView(Plugin p){
    return (p.getType() < CONTROLER_VIEW_LIMIT);
  }

  /**
   * test if the given s type is a view type
   * @param type the type to test
   * @return true if this type is a controler, false otherwise
   */
  public static boolean isView(int type){
    return (type < CONTROLER_VIEW_LIMIT);
  }

  /**
   * test if the given plugin's type is manager
   * @param p the plugin to test
   * @return true if the type is a manager, false otherwise
   */
  public static boolean isManager(Plugin p){
    return (p.getType() == MANAGER);
  }

  /**
   * test if the given plugin's type is AbstractTopManager
   * @param p the plugin to test
   * @return true if the tpye is a AbstractTopManager, false otherwise
   */
  public static boolean isTopManager(Plugin p){
    return (p.getType() == TOP_MANAGER);
  }

  /**
   * return the type name of the given type
   * @param type
   * @return the name of the type
   */
  public static String getName(int type){
    String res = (String) typeNames.get(new Integer(type));
    if (res == null) throw new NullPointerException();
    return res;
  }
}
