package fr.unice.javinspector.util;

/**
 * <p>Title: Tree</p>
 * <p>Description: A basic implementation of a biColored-tree of String</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: UNICE</p>
 * @author Zerbib Gabriel - Charbit Lucas - Galbois Xavier - Gahide Jrme
 * @version 1.0
 */
public class Tree {

  private String value = null;

  private Tree leftChild;

  private Tree rightChild;

  /**
   * Construct a new empty Tree.
   */
  public Tree() {
    leftChild = null;
    rightChild = null;
  }

  /**
   *
   * @return the value root of the tree or the leaf value.
   * @throws Exception if the tree is empty
   */
  public String getValue()
      throws Exception{
    if(! is_empty())
      return value;
    else
      throw new Exception("This tree is empty : can not return the root value.");
  }

  /**
   * Set the root or leaf value for this tree.
   * @param value the string to set.
   */
  public void setValue(String value){
    this.value = value;
  }

  /**
   * the tree is empty if the left child, the right child and the value are null.
   * @return true if the tree is empty.
   */
  public boolean is_empty(){
    return value == null && leftChild == null && rightChild == null;
  }

  /**
   * the tree is a leaf if the left child, the right child are null but not the value.
   * @return true if the tree is a leaf.
   */
  public boolean is_leaf(){
    return value != null && leftChild == null && rightChild == null;
  }

  /**
   * This function return the leftChild.
   * NB : null if there is no leftChild.
   * @return the leftChild
   */
  public Tree getLeftChild(){
      return leftChild;
  }

  /**
   * This function return the rightChild.
   * NB : null if there is no rightChild.
   * @return the rightChild
   */
  public Tree getRightChild(){
    return rightChild;
  }

  /**
   * This procedure set the leftChild of the tree.
   * @param tree the to get the leftChild
   * NB : the tree must not to be null.
   */
  public void setLeftChild(Tree tree){
    this.leftChild = tree;
  }

  /**
   * This procedure set the rightChild of the tree.
   * @param tree the to get the rightChild
   * NB : the tree must not to be null.
   */
  public void setRightChild(Tree tree){
    this.rightChild = tree;
  }

  /**
   * This function do the navigation by the left on the tree t.
   * @param t the tree to course.
   * @return the String retpresentation of the tree by the left course.
   * Ex: (+ 4 (- 3 2)) == 4 + (3 - 2)
   */
  public static String leftCourse(Tree t){
    String left  = "";
    String right = "";
    try{
      if( t.is_empty() ) return "";
      else if( t.is_leaf() ) return t.getValue();
      // node
      else{
        if( t.getLeftChild() != null )
          left = leftCourse(t.getLeftChild());
        if( t.getRightChild() != null )
          right = leftCourse(t.getRightChild());
        return left + t.getValue() + right;
      }


    }catch(Exception e){
      System.err.println("Error during the course occur aborting");
      System.out.println(e.getMessage());
    }
    return "";

  }

  /**
   * @return a string representation of the tree (node leaf ).
   */
  public String toString(){
    if( is_empty())
      return "I'am an empty tree";
    else if( is_leaf() )
      return "I'am a leaf and may value is : " + value;
    else
      return "I'am a tree and may value is : " + value;
  }



}
